@@ -8,18 +8,18 @@ from PyQt5.QtGui import QIcon, QDrag, QPixmap, QRegion, QDropEvent, QTextCursor,
     QResizeEvent
 from PyQt5.QtWidgets import QFrame, QMessageBox, QMenu, QWidget, QUndoStack, QCheckBox, QApplication, qApp
 
-from urh import constants
+from urh import settings
 from urh.controller.dialogs.AdvancedModulationOptionsDialog import AdvancedModulationOptionsDialog
 from urh.controller.dialogs.FilterDialog import FilterDialog
 from urh.controller.dialogs.SendDialog import SendDialog
 from urh.controller.dialogs.SignalDetailsDialog import SignalDetailsDialog
 from urh.signalprocessing.Filter import Filter, FilterType
+from urh.signalprocessing.IQArray import IQArray
 from urh.signalprocessing.ProtocolAnalyzer import ProtocolAnalyzer
 from urh.signalprocessing.Signal import Signal
 from urh.signalprocessing.Spectrogram import Spectrogram
 from urh.ui.actions.ChangeSignalParameter import ChangeSignalParameter
 from urh.ui.actions.EditSignalAction import EditSignalAction, EditAction
-from urh.ui.painting.LegendScene import LegendScene
 from urh.ui.painting.SignalSceneManager import SignalSceneManager
 from urh.ui.ui_signal_frame import Ui_SignalFrame
 from urh.util import FileOperator, util
@@ -119,7 +119,6 @@ class SignalFrame(QFrame):
             else:
                 self.ui.lSignalTyp.setText("Complex Signal")
 
-            self.ui.gvLegend.hide()
             self.ui.lineEditSignalName.setText(self.signal.name)
             self.ui.lSamplesInView.setText("{0:,}".format(self.signal.num_samples))
             self.ui.lSamplesTotal.setText("{0:,}".format(self.signal.num_samples))
@@ -132,8 +131,12 @@ class SignalFrame(QFrame):
 
             self.scene_manager = SignalSceneManager(self.signal, self)
             self.ui.gvSignal.scene_manager = self.scene_manager
+            self.scene_manager.scene.setParent(self.ui.gvSignal)
             self.ui.gvSignal.setScene(self.scene_manager.scene)
 
+            self.ui.spinBoxCenterSpacing.setValue(self.signal.center_spacing)
+            self.ui.spinBoxBitsPerSymbol.setValue(self.signal.bits_per_symbol)
+
             self.jump_sync = True
             self.on_btn_show_hide_start_end_clicked()
 
@@ -142,7 +145,6 @@ class SignalFrame(QFrame):
             self.set_protocol_visibility()
 
             self.ui.chkBoxShowProtocol.setChecked(True)
-            self.set_qad_tooltip(self.signal.noise_threshold)
             self.ui.btnSaveSignal.hide()
 
             self.show_protocol(refresh=False)
@@ -152,6 +154,8 @@ class SignalFrame(QFrame):
             self.set_empty_frame_visibilities()
             self.create_connects()
 
+        self.set_center_spacing_visibility()
+
     @property
     def spectrogram_is_active(self) -> bool:
         return self.ui.stackedWidget.currentWidget() == self.ui.pageSpectrogram
@@ -174,20 +178,21 @@ class SignalFrame(QFrame):
         if self.signal is not None:
             self.ui.gvSignal.save_clicked.connect(self.save_signal)
 
-            self.signal.bit_len_changed.connect(self.ui.spinBoxInfoLen.setValue)
-            self.signal.qad_center_changed.connect(self.on_signal_qad_center_changed)
+            self.signal.samples_per_symbol_changed.connect(self.ui.spinBoxSamplesPerSymbol.setValue)
+            self.signal.center_changed.connect(self.on_signal_center_changed)
             self.signal.noise_threshold_changed.connect(self.on_noise_threshold_changed)
-            self.signal.modulation_type_changed.connect(self.ui.cbModulationType.setCurrentIndex)
+            self.signal.modulation_type_changed.connect(self.ui.cbModulationType.setCurrentText)
             self.signal.tolerance_changed.connect(self.ui.spinBoxTolerance.setValue)
             self.signal.protocol_needs_update.connect(self.refresh_protocol)
             self.signal.data_edited.connect(self.on_signal_data_edited)  # Crop/Delete Mute etc.
+            self.signal.bits_per_symbol_changed.connect(self.ui.spinBoxBitsPerSymbol.setValue)
+            self.signal.center_spacing_changed.connect(self.on_signal_center_spacing_changed)
 
             self.signal.sample_rate_changed.connect(self.on_signal_sample_rate_changed)
 
             self.signal.saved_status_changed.connect(self.on_signal_data_changed_before_save)
             self.ui.btnSaveSignal.clicked.connect(self.save_signal)
             self.signal.name_changed.connect(self.ui.lineEditSignalName.setText)
-            self.ui.gvLegend.resized.connect(self.on_gv_legend_resized)
 
             self.ui.gvSignal.selection_width_changed.connect(self.start_proto_selection_timer)
             self.ui.gvSignal.sel_area_start_end_changed.connect(self.start_proto_selection_timer)
@@ -209,8 +214,7 @@ class SignalFrame(QFrame):
         self.ui.gvSignal.sel_area_start_end_changed.connect(self.update_selection_area)
         self.ui.gvSpectrogram.sel_area_start_end_changed.connect(self.update_selection_area)
         self.ui.gvSpectrogram.selection_height_changed.connect(self.update_number_selected_samples)
-        self.ui.gvSignal.sep_area_changed.connect(self.set_qad_center)
-        self.ui.gvSignal.sep_area_moving.connect(self.update_legend)
+        self.ui.gvSignal.sep_area_changed.connect(self.set_center)
 
         self.ui.sliderYScale.valueChanged.connect(self.on_slider_y_scale_value_changed)
         self.ui.spinBoxXZoom.valueChanged.connect(self.on_spinbox_x_zoom_value_changed)
@@ -222,7 +226,7 @@ class SignalFrame(QFrame):
         self.proto_selection_timer.timeout.connect(self.update_number_selected_samples)
 
         self.ui.cbSignalView.currentIndexChanged.connect(self.on_cb_signal_view_index_changed)
-        self.ui.cbModulationType.currentIndexChanged.connect(self.on_combobox_modulation_type_index_changed)
+        self.ui.cbModulationType.currentTextChanged.connect(self.on_combobox_modulation_type_text_changed)
         self.ui.cbProtoView.currentIndexChanged.connect(self.on_combo_box_proto_view_index_changed)
 
         self.ui.chkBoxShowProtocol.stateChanged.connect(self.set_protocol_visibility)
@@ -237,27 +241,38 @@ class SignalFrame(QFrame):
         self.ui.spinBoxSelectionStart.valueChanged.connect(self.on_spinbox_selection_start_value_changed)
         self.ui.spinBoxSelectionEnd.valueChanged.connect(self.on_spinbox_selection_end_value_changed)
         self.ui.spinBoxCenterOffset.editingFinished.connect(self.on_spinbox_center_editing_finished)
+        self.ui.spinBoxCenterSpacing.valueChanged.connect(self.on_spinbox_spacing_value_changed)
+        self.ui.spinBoxCenterSpacing.editingFinished.connect(self.on_spinbox_spacing_editing_finished)
         self.ui.spinBoxTolerance.editingFinished.connect(self.on_spinbox_tolerance_editing_finished)
         self.ui.spinBoxNoiseTreshold.editingFinished.connect(self.on_spinbox_noise_threshold_editing_finished)
-        self.ui.spinBoxInfoLen.editingFinished.connect(self.on_spinbox_infolen_editing_finished)
+        self.ui.spinBoxSamplesPerSymbol.editingFinished.connect(self.on_spinbox_samples_per_symbol_editing_finished)
+        self.ui.spinBoxBitsPerSymbol.editingFinished.connect(self.on_spinbox_bits_per_symbol_editing_finished)
 
     def refresh_signal_information(self, block=True):
         self.ui.spinBoxTolerance.blockSignals(block)
         self.ui.spinBoxCenterOffset.blockSignals(block)
-        self.ui.spinBoxInfoLen.blockSignals(block)
+        self.ui.spinBoxSamplesPerSymbol.blockSignals(block)
         self.ui.spinBoxNoiseTreshold.blockSignals(block)
+        self.ui.spinBoxBitsPerSymbol.blockSignals(block)
+        self.ui.spinBoxCenterSpacing.blockSignals(block)
 
         self.ui.spinBoxTolerance.setValue(self.signal.tolerance)
-        self.ui.spinBoxCenterOffset.setValue(self.signal.qad_center)
-        self.ui.spinBoxInfoLen.setValue(self.signal.bit_len)
-        self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold)
-        self.ui.cbModulationType.setCurrentIndex(self.signal.modulation_type)
+        self.ui.spinBoxCenterOffset.setValue(self.signal.center)
+        self.ui.spinBoxSamplesPerSymbol.setValue(self.signal.samples_per_symbol)
+        self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold_relative)
+        self.ui.cbModulationType.setCurrentText(self.signal.modulation_type)
         self.ui.btnAdvancedModulationSettings.setVisible(self.ui.cbModulationType.currentText() == "ASK")
+        self.ui.spinBoxCenterSpacing.setValue(self.signal.center_spacing)
+        self.ui.spinBoxBitsPerSymbol.setValue(self.signal.bits_per_symbol)
 
         self.ui.spinBoxTolerance.blockSignals(False)
         self.ui.spinBoxCenterOffset.blockSignals(False)
-        self.ui.spinBoxInfoLen.blockSignals(False)
+        self.ui.spinBoxSamplesPerSymbol.blockSignals(False)
         self.ui.spinBoxNoiseTreshold.blockSignals(False)
+        self.ui.spinBoxCenterSpacing.blockSignals(False)
+        self.ui.spinBoxBitsPerSymbol.blockSignals(False)
+
+        self.set_center_spacing_visibility()
 
     def set_empty_frame_visibilities(self):
         for widget in dir(self.ui):
@@ -266,6 +281,8 @@ class SignalFrame(QFrame):
                                                 self.ui.btnCloseSignal, self.ui.lineEditSignalName):
                 w.hide()
 
+        self.adjustSize()
+
     def cancel_filtering(self):
         self.filter_abort_wanted = True
 
@@ -283,8 +300,8 @@ class SignalFrame(QFrame):
             power_str = "-\u221e"  # minus infinity
             if start < end:
                 max_window_size = 10 ** 5
-                step_size = int(math.ceil((end-start)/max_window_size))
-                power = np.mean(np.abs(self.signal.data[start:end:step_size]))
+                step_size = int(math.ceil((end - start) / max_window_size))
+                power = np.mean(self.signal.iq_array.subarray(start, end, step_size).magnitudes_normalized)
                 if power > 0:
                     power_str = Formatter.big_value_with_suffix(10 * np.log10(power), 2)
 
@@ -295,7 +312,8 @@ class SignalFrame(QFrame):
             self.ui.labelRSSI.setText("")
 
     def change_signal_name(self):
-        self.signal.name = self.ui.lineEditSignalName.text()
+        if self.signal is not None:
+            self.signal.name = self.ui.lineEditSignalName.text()
 
     def __set_spectrogram_adjust_widgets_visibility(self):
         self.ui.labelFFTWindowSize.setVisible(self.ui.cbSignalView.currentIndex() == 2)
@@ -333,6 +351,8 @@ class SignalFrame(QFrame):
             gv.scale(1, yscale / current_factor)
             x, w = gv.view_rect().x(), gv.view_rect().width()
             gv.centerOn(x + w / 2, gv.y_center)
+            if gv.scene_type == 1:
+                gv.scene().redraw_legend()
         except ZeroDivisionError:
             pass
 
@@ -385,8 +405,7 @@ class SignalFrame(QFrame):
             Errors.empty_selection()
 
     def my_close(self):
-        settings = constants.SETTINGS
-        not_show = settings.value('not_show_close_dialog', False, type=bool)
+        not_show = settings.read('not_show_close_dialog', False, type=bool)
 
         if not not_show:
             cb = QCheckBox("Do not show this again.")
@@ -399,7 +418,7 @@ class SignalFrame(QFrame):
             reply = msgbox.exec()
 
             not_show_again = bool(cb.isChecked())
-            settings.setValue("not_show_close_dialog", not_show_again)
+            settings.write("not_show_close_dialog", not_show_again)
             self.not_show_again_changed.emit()
             if reply != QMessageBox.Yes:
                 return
@@ -413,17 +432,11 @@ class SignalFrame(QFrame):
             self.save_signal_as()
 
     def save_signal_as(self):
-        if self.signal.filename:
-            initial_name = self.signal.filename
-        else:
-            initial_name = self.signal.name.replace(" ", "-").replace(",", ".").replace(".", "_") + ".complex"
-
-        filename = FileOperator.get_save_file_name(initial_name, wav_only=self.signal.wav_mode)
-        if filename:
-            try:
-                self.signal.save_as(filename)
-            except Exception as e:
-                QMessageBox.critical(self, self.tr("Error saving signal"), e.args[0])
+        try:
+            FileOperator.save_data_dialog(self.signal.name, self.signal.iq_array, self.signal.sample_rate,
+                                          self.signal.wav_mode)
+        except Exception as e:
+            Errors.exception(e)
 
     def export_demodulated(self):
         try:
@@ -440,16 +453,13 @@ class SignalFrame(QFrame):
                 if filename.endswith(".wav"):
                     data = self.signal.qad.astype(np.float32)
                     data /= np.max(np.abs(data))
-                data = FileOperator.convert_data_to_format(data, filename)
-                FileOperator.save_data(data, filename, self.signal.sample_rate, num_channels=1)
+                FileOperator.save_data(IQArray(data, skip_conversion=True), filename, self.signal.sample_rate,
+                                       num_channels=1)
                 self.unsetCursor()
             except Exception as e:
                 QMessageBox.critical(self, self.tr("Error exporting demodulated data"), e.args[0])
 
     def draw_signal(self, full_signal=False):
-        gv_legend = self.ui.gvLegend
-        gv_legend.y_sep = -self.signal.qad_center
-
         self.scene_manager.scene_type = self.ui.cbSignalView.currentIndex()
         self.scene_manager.init_scene()
         if full_signal:
@@ -457,14 +467,7 @@ class SignalFrame(QFrame):
         else:
             self.ui.gvSignal.redraw_view()
 
-        legend = LegendScene()
-        legend.setBackgroundBrush(constants.BGCOLOR)
-        legend.setSceneRect(0, self.scene_manager.scene.sceneRect().y(), gv_legend.width(),
-                            self.scene_manager.scene.sceneRect().height())
-        legend.draw_one_zero_arrows(-self.signal.qad_center)
-        gv_legend.setScene(legend)
-
-        self.ui.gvSignal.y_sep = -self.signal.qad_center
+        self.ui.gvSignal.y_sep = -self.signal.center
 
     def restore_protocol_selection(self, sel_start, sel_end, start_message, end_message, old_protoview):
         if old_protoview == self.proto_view:
@@ -503,6 +506,9 @@ class SignalFrame(QFrame):
 
     def update_protocol(self):
         self.ui.txtEdProto.setEnabled(False)
+        self.ui.txtEdProto.setText("Demodulating...")
+        qApp.processEvents()
+
         self.proto_analyzer.get_protocol_from_signal()
 
     def show_protocol(self, old_view=-1, refresh=False):
@@ -572,7 +578,8 @@ class SignalFrame(QFrame):
         window_size = 2 ** self.ui.sliderFFTWindowSize.value()
         data_min, data_max = self.ui.sliderSpectrogramMin.value(), self.ui.sliderSpectrogramMax.value()
 
-        redraw_needed = self.ui.gvSpectrogram.scene_manager.set_parameters(self.signal.data, window_size=window_size,
+        redraw_needed = self.ui.gvSpectrogram.scene_manager.set_parameters(self.signal.iq_array.data,
+                                                                           window_size=window_size,
                                                                            data_min=data_min, data_max=data_max)
         self.ui.gvSpectrogram.scene_manager.update_scene_rect()
 
@@ -599,7 +606,6 @@ class SignalFrame(QFrame):
             self.proto_analyzer.eliminate()
             self.ui.gvSignal.scene_manager.eliminate()
 
-        self.ui.gvLegend.eliminate()
         self.ui.gvSignal.eliminate()
         self.ui.gvSpectrogram.eliminate()
 
@@ -644,14 +650,20 @@ class SignalFrame(QFrame):
         start = self.ui.gvSignal.selection_area.x
         end = start + self.ui.gvSignal.selection_area.width
 
-        new_thresh = self.signal.calc_noise_threshold(start, end)
+        new_thresh = self.signal.calc_relative_noise_threshold_from_range(start, end)
         self.ui.spinBoxNoiseTreshold.setValue(new_thresh)
         self.ui.spinBoxNoiseTreshold.editingFinished.emit()
         self.unsetCursor()
 
+    @pyqtSlot(float)
+    def on_signal_center_spacing_changed(self, value: float):
+        self.ui.spinBoxCenterSpacing.setValue(value)
+        if self.ui.gvSignal.scene_type == 1:
+            self.ui.gvSignal.scene().redraw_legend()
+
     @pyqtSlot()
     def on_noise_threshold_changed(self):
-        self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold)
+        self.ui.spinBoxNoiseTreshold.setValue(self.signal.noise_threshold_relative)
         minimum = self.signal.noise_min_plot
         maximum = self.signal.noise_max_plot
         if self.ui.cbSignalView.currentIndex() == 0:
@@ -686,15 +698,6 @@ class SignalFrame(QFrame):
         self.ui.txtEdProto.setEnabled(True)
         self.ui.txtEdProto.setHtml(self.proto_analyzer.plain_to_html(self.proto_view))
 
-    @pyqtSlot(float)
-    def update_legend(self, y_sep):
-        if self.ui.gvLegend.isVisible():
-            self.ui.gvLegend.y_sep = y_sep
-            self.ui.gvLegend.refresh()
-        self.ui.spinBoxCenterOffset.blockSignals(True)
-        self.ui.spinBoxCenterOffset.setValue(-y_sep)
-        self.ui.spinBoxCenterOffset.blockSignals(False)
-
     @pyqtSlot()
     def handle_protocol_sync_changed(self):
         self.sync_protocol = self.ui.chkBoxSyncSelection.isChecked()
@@ -731,13 +734,6 @@ class SignalFrame(QFrame):
             self.ui.gvSignal.redraw_view(reinitialize=True)
             self.ui.labelRSSI.show()
 
-            if self.ui.cbSignalView.currentIndex() == 1:
-                self.ui.gvLegend.y_scene = self.scene_manager.scene.sceneRect().y()
-                self.ui.gvLegend.scene_height = self.scene_manager.scene.sceneRect().height()
-                self.ui.gvLegend.refresh()
-            else:
-                self.ui.gvLegend.hide()
-
             self.ui.gvSignal.auto_fit_view()
             self.ui.gvSignal.refresh_selection_area()
             qApp.processEvents()
@@ -749,14 +745,21 @@ class SignalFrame(QFrame):
 
     @pyqtSlot()
     def on_btn_autodetect_clicked(self):
-        self.signal.auto_detect(detect_modulation=self.detect_modulation_action.isChecked(),
-                                detect_noise=self.detect_noise_action.isChecked())
+        self.ui.btnAutoDetect.setEnabled(False)
+        self.setCursor(Qt.WaitCursor)
+        success = self.signal.auto_detect(detect_modulation=self.detect_modulation_action.isChecked(),
+                                          detect_noise=self.detect_noise_action.isChecked())
+        self.ui.btnAutoDetect.setEnabled(True)
+        self.unsetCursor()
+        if not success:
+            Errors.generic_error(self.tr("Autodetection failed"),
+                                 self.tr("Failed to autodetect parameters for this signal."))
 
     @pyqtSlot()
     def on_btn_replay_clicked(self):
         project_manager = self.project_manager
         try:
-            dialog = SendDialog(project_manager, modulated_data=self.signal.data, parent=self)
+            dialog = SendDialog(project_manager, modulated_data=self.signal.iq_array, parent=self)
         except OSError as e:
             logger.error(repr(e))
             return
@@ -791,7 +794,7 @@ class SignalFrame(QFrame):
         self.show_protocol(old_view=old_view)
 
     @pyqtSlot(float)
-    def set_qad_center(self, th):
+    def set_center(self, th):
         self.ui.spinBoxCenterOffset.setValue(th)
         self.ui.spinBoxCenterOffset.editingFinished.emit()
 
@@ -985,39 +988,25 @@ class SignalFrame(QFrame):
         self.draw_signal(draw_full_signal)
 
         self.__set_samples_in_view()
-
         self.update_number_selected_samples()
-
-        self.set_qad_tooltip(self.signal.noise_threshold)
         self.on_slider_y_scale_value_changed()
 
     @pyqtSlot(float)
-    def on_signal_qad_center_changed(self, qad_center):
-        self.ui.gvSignal.y_sep = -qad_center
-        self.ui.gvLegend.y_sep = -qad_center
+    def on_signal_center_changed(self, center):
+        self.ui.gvSignal.y_sep = -center
 
         if self.ui.cbSignalView.currentIndex() > 0:
-            self.scene_manager.scene.draw_sep_area(-qad_center)
-            self.ui.gvLegend.refresh()
+            self.scene_manager.scene.draw_sep_area(-self.signal.center_thresholds)
         self.ui.spinBoxCenterOffset.blockSignals(False)
-        self.ui.spinBoxCenterOffset.setValue(qad_center)
+        self.ui.spinBoxCenterOffset.setValue(center)
 
     def on_spinbox_noise_threshold_editing_finished(self):
-        if self.signal is not None and self.signal.noise_threshold != self.ui.spinBoxNoiseTreshold.value():
+        if self.signal is not None and self.signal.noise_threshold_relative != self.ui.spinBoxNoiseTreshold.value():
             noise_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
-                                                 parameter_name="noise_threshold",
+                                                 parameter_name="noise_threshold_relative",
                                                  parameter_value=self.ui.spinBoxNoiseTreshold.value())
             self.undo_stack.push(noise_action)
 
-    def set_qad_tooltip(self, noise_threshold):
-        self.ui.cbSignalView.setToolTip(
-            "<html><head/><body><p>Choose the view of your signal: Analog, Demodulated or Spectrogram.</p>"
-            "<p>The quadrature demodulation uses a <b>threshold of magnitude,</b> to <b>supress noise</b>. "
-            "All samples with a magnitude lower than this threshold will be eliminated "
-            "(set to <i>-127</i>) after demodulation.</p>"
-            "<p>Tune this value by selecting a <i>noisy area</i> and mark it as noise using <b>context menu</b>.</p>"
-            "<p>Current noise threshold is: <b>" + str(noise_threshold) + "</b></p></body></html>")
-
     def contextMenuEvent(self, event: QContextMenuEvent):
         if self.signal is None:
             return
@@ -1038,7 +1027,7 @@ class SignalFrame(QFrame):
 
     def show_modulation_type(self):
         self.ui.cbModulationType.blockSignals(True)
-        self.ui.cbModulationType.setCurrentIndex(self.signal.modulation_type)
+        self.ui.cbModulationType.setCurrentText(self.signal.modulation_type)
         self.ui.cbModulationType.blockSignals(False)
 
     def on_participant_changed(self):
@@ -1054,25 +1043,27 @@ class SignalFrame(QFrame):
         if not self.spectrogram_is_active:
             self.ui.gvSignal.zoom(new_width / old_width, zoom_to_mouse_cursor=False)
 
+    def set_center_spacing_visibility(self):
+        visible = self.ui.spinBoxBitsPerSymbol.value() > 1
+        self.ui.spinBoxCenterSpacing.setVisible(visible)
+        self.ui.lCenterSpacing.setVisible(visible)
+
     @pyqtSlot()
     def on_info_btn_clicked(self):
         sdc = SignalDetailsDialog(self.signal, self)
         sdc.show()
 
-    @pyqtSlot(int)
-    def on_combobox_modulation_type_index_changed(self, index: int):
-        if index != self.signal.modulation_type:
+    @pyqtSlot(str)
+    def on_combobox_modulation_type_text_changed(self, txt: str):
+        if txt != self.signal.modulation_type:
             modulation_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
                                                       parameter_name="modulation_type",
-                                                      parameter_value=index)
+                                                      parameter_value=txt)
 
             self.undo_stack.push(modulation_action)
 
             if self.ui.cbSignalView.currentIndex() == 1:
                 self.scene_manager.init_scene()
-                self.ui.gvLegend.y_scene = self.scene_manager.scene.sceneRect().y()
-                self.ui.gvLegend.scene_height = self.scene_manager.scene.sceneRect().height()
-                self.ui.gvLegend.refresh()
                 self.on_slider_y_scale_value_changed()
 
         self.ui.btnAdvancedModulationSettings.setVisible(self.ui.cbModulationType.currentText() == "ASK")
@@ -1099,13 +1090,6 @@ class SignalFrame(QFrame):
 
         self.ui.lineEditSignalName.setFont(font)
 
-    @pyqtSlot()
-    def on_gv_legend_resized(self):
-        if self.ui.gvLegend.isVisible():
-            self.ui.gvLegend.y_zoom_factor = self.ui.gvSignal.transform().m22()
-            self.ui.gvLegend.refresh()
-            self.ui.gvLegend.translate(0, 1)  # Resize verschiebt sonst Pfeile
-
     @pyqtSlot()
     def on_btn_show_hide_start_end_clicked(self):
         show = self.ui.btnShowHideStartEnd.isChecked()
@@ -1133,23 +1117,58 @@ class SignalFrame(QFrame):
             self.ui.spinBoxTolerance.blockSignals(False)
 
     @pyqtSlot()
-    def on_spinbox_infolen_editing_finished(self):
-        if self.signal.bit_len != self.ui.spinBoxInfoLen.value():
-            self.ui.spinBoxInfoLen.blockSignals(True)
-            bitlen_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
-                                                  parameter_name="bit_len",
-                                                  parameter_value=self.ui.spinBoxInfoLen.value())
-            self.undo_stack.push(bitlen_action)
-            self.ui.spinBoxInfoLen.blockSignals(False)
+    def on_spinbox_samples_per_symbol_editing_finished(self):
+        if self.signal.samples_per_symbol != self.ui.spinBoxSamplesPerSymbol.value():
+            self.ui.spinBoxSamplesPerSymbol.blockSignals(True)
+            action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
+                                           parameter_name="samples_per_symbol",
+                                           parameter_value=self.ui.spinBoxSamplesPerSymbol.value())
+            self.undo_stack.push(action)
+            self.ui.spinBoxSamplesPerSymbol.blockSignals(False)
+
+    @pyqtSlot()
+    def on_spinbox_bits_per_symbol_editing_finished(self):
+        if self.signal.bits_per_symbol != self.ui.spinBoxBitsPerSymbol.value():
+            self.ui.spinBoxBitsPerSymbol.blockSignals(True)
+            bits_per_symbol_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
+                                                           parameter_name="bits_per_symbol",
+                                                           parameter_value=self.ui.spinBoxBitsPerSymbol.value())
+            self.undo_stack.push(bits_per_symbol_action)
+            self.ui.spinBoxBitsPerSymbol.blockSignals(False)
+
+            if self.ui.gvSignal.scene_type == 1:
+                self.ui.gvSignal.scene().draw_sep_area(-self.signal.center_thresholds)
+
+            self.set_center_spacing_visibility()
 
     @pyqtSlot()
     def on_spinbox_center_editing_finished(self):
-        if self.signal.qad_center != self.ui.spinBoxCenterOffset.value():
+        if self.signal.center != self.ui.spinBoxCenterOffset.value():
             self.ui.spinBoxCenterOffset.blockSignals(True)
             center_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
-                                                  parameter_name="qad_center",
+                                                  parameter_name="center",
                                                   parameter_value=self.ui.spinBoxCenterOffset.value())
             self.undo_stack.push(center_action)
+            self.ui.spinBoxCenterOffset.blockSignals(False)
+
+    @pyqtSlot()
+    def on_spinbox_spacing_value_changed(self):
+        if self.ui.gvSignal.scene_type == 1:
+            thresholds = self.signal.get_thresholds_for_center(self.signal.center, self.ui.spinBoxCenterSpacing.value())
+            self.ui.gvSignal.scene().draw_sep_area(-thresholds)
+
+    @pyqtSlot()
+    def on_spinbox_spacing_editing_finished(self):
+        if self.signal.center_spacing != self.ui.spinBoxCenterSpacing.value():
+            self.ui.spinBoxCenterSpacing.blockSignals(True)
+            center_spacing_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
+                                                          parameter_name="center_spacing",
+                                                          parameter_value=self.ui.spinBoxCenterSpacing.value())
+            self.undo_stack.push(center_spacing_action)
+            self.ui.spinBoxCenterSpacing.blockSignals(False)
+
+            if self.ui.gvSignal.scene_type == 1:
+                self.ui.gvSignal.scene().draw_sep_area(-self.signal.center_thresholds)
 
     @pyqtSlot()
     def refresh(self, draw_full_signal=False):
@@ -1196,7 +1215,8 @@ class SignalFrame(QFrame):
         QApplication.instance().setOverrideCursor(Qt.WaitCursor)
         filter_bw = Filter.read_configured_filter_bw()
         filtered = Array("f", 2 * self.signal.num_samples)
-        p = Process(target=perform_filter, args=(filtered, self.signal.data, f_low, f_high, filter_bw))
+        p = Process(target=perform_filter,
+                    args=(filtered, self.signal.iq_array.as_complex64(), f_low, f_high, filter_bw))
         p.daemon = True
         p.start()
 
@@ -1241,17 +1261,22 @@ class SignalFrame(QFrame):
 
     @pyqtSlot(int)
     def on_message_length_divisor_edited(self, message_length_divisor: int):
-        if self.signal.pause_threshold != message_length_divisor:
+        if self.signal.message_length_divisor != message_length_divisor:
             message_length_divisor_action = ChangeSignalParameter(signal=self.signal, protocol=self.proto_analyzer,
                                                                   parameter_name="message_length_divisor",
                                                                   parameter_value=message_length_divisor)
             self.undo_stack.push(message_length_divisor_action)
 
-    @pyqtSlot()
-    def on_btn_advanced_modulation_settings_clicked(self):
-        dialog = AdvancedModulationOptionsDialog(self.signal.pause_threshold, self.signal.message_length_divisor, parent=self)
+    def get_advanced_modulation_settings_dialog(self):
+        dialog = AdvancedModulationOptionsDialog(self.signal.pause_threshold, self.signal.message_length_divisor,
+                                                 parent=self)
         dialog.pause_threshold_edited.connect(self.on_pause_threshold_edited)
         dialog.message_length_divisor_edited.connect(self.on_message_length_divisor_edited)
+        return dialog
+
+    @pyqtSlot()
+    def on_btn_advanced_modulation_settings_clicked(self):
+        dialog = self.get_advanced_modulation_settings_dialog()
         dialog.exec_()
 
     @pyqtSlot()
@@ -1271,7 +1296,6 @@ class SignalFrame(QFrame):
                                                                           filename=filename,
                                                                           include_amplitude=filename.endswith(".fta"))
         except Exception as e:
-            logger.exception(e)
-            Errors.generic_error("Failed to export spectrogram", str(e))
+            Errors.exception(e)
         finally:
             QApplication.restoreOverrideCursor()
